<?php

namespace Drupal\path_tools;

use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\ParamConverter\ParamNotConvertedException;
use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Routing\AccessAwareRouterInterface;
use Drupal\Core\Routing\RequestContext;
use Drupal\Core\Routing\RouteMatch;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\Routing\Exception\MethodNotAllowedException;
use Symfony\Component\Routing\Exception\ResourceNotFoundException;

/**
 * Provides services to determine parent entities from path.
 */
class ParentEntityFromPathService implements ParentEntityFromPathServiceInterface {

  /**
   * Constructs a ParentEntity object.
   *
   * @param \Drupal\Core\Routing\RequestContext $requestContext
   *   The request context.
   * @param \Drupal\Core\Access\AccessManagerInterface $accessManager
   *   The access manager.
   * @param \Drupal\Core\Path\PathMatcherInterface $pathMatcher
   *   The path matcher.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\PathProcessor\InboundPathProcessorInterface $pathProcessor
   *   The path processor.
   * @param \Drupal\Core\Routing\AccessAwareRouterInterface $accessAwareRouter
   *   The access aware router.
   * @param \Drupal\Core\Path\CurrentPathStack $currentPath
   *   The current path.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   */
  public function __construct(
    protected readonly RequestContext $requestContext,
    protected readonly AccessManagerInterface $accessManager,
    protected readonly PathMatcherInterface $pathMatcher,
    protected readonly ConfigFactoryInterface $configFactory,
    protected readonly InboundPathProcessorInterface $pathProcessor,
    protected readonly AccessAwareRouterInterface $accessAwareRouter,
    protected readonly CurrentPathStack $currentPath,
    protected readonly AccountProxyInterface $currentUser
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getPathBasedParentEntity(): ?ContentEntityInterface {
    $entities = $this->getPathBasedParentEntities();
    return !empty($entities) ? reset($entities) : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getPathBasedParentEntities(): array {
    $entities = [];

    // General path-based logic. Use the actual request path, prior to
    // resolving path aliases, so the logic can be defined by simply
    // creating a hierarchy of path aliases.
    $path = trim($this->requestContext->getPathInfo(), '/');
    $path_elements = explode('/', $path);
    while (count($path_elements) > 1) {
      array_pop($path_elements);
      // Copy the path elements for up-casting.
      $route_request = $this->getRequestForPath('/' . implode('/', $path_elements), []);
      if ($route_request) {
        $route_match = RouteMatch::createFromRequest($route_request);
        $access = $this->accessManager->check($route_match, $this->currentUser, NULL, TRUE);
        if ($access->isAllowed()) {
          // We assume that the first entity from route is the canoncial entity.
          foreach ($route_match->getParameters() as $entity) {
            if ($entity instanceof ContentEntityInterface) {
              $entities[] = $entity;
            }
          }
        }
      }
    }

    return $entities;
  }

  /**
   * {@inheritdoc}
   */
  public function getRequestForPath($path, array $exclude): ?Request {
    if (!empty($exclude[$path])) {
      return NULL;
    }
    $request = Request::create($path);
    // Performance optimization: set a short accept header to reduce overhead in
    // AcceptHeaderMatcher when matching the request.
    $request->headers->set('Accept', 'text/html');
    // Find the system path by resolving aliases, language prefix, etc.
    $processed = $this->pathProcessor->processInbound($path, $request);
    if (empty($processed) || !empty($exclude[$processed])) {
      // This resolves to the front page, which we already add.
      return NULL;
    }
    $this->currentPath->setPath($processed, $request);
    // Attempt to match this path to provide a fully built request.
    try {
      $request->attributes->add($this->accessAwareRouter->matchRequest($request));
      return $request;
    } catch (ParamNotConvertedException $e) {
      return NULL;
    } catch (ResourceNotFoundException $e) {
      return NULL;
    } catch (MethodNotAllowedException $e) {
      return NULL;
    } catch (AccessDeniedHttpException $e) {
      return NULL;
    }
  }

}
